package dw.samples.ldapcontrols;

/**
 * A Java control class to represent response controls of the Password Policy
 * Control.
 * 
 * It extends the request control with the control specific data. This is
 * accomplished by the attributes timeBeforeExpiration, graceLoginsRemaining and
 * errorCode, all of which have getters and setters. getEncodedValue returns the
 * unchanged value of the response control as byte array, the other getters
 * allow convenient access to the state. The method toString provides a textual
 * representation of the control value. In particular, it includes the message
 * text for any error code (e.g. "password expired") that might be present.
 * 
 * developerWorks, Take advantage of IBM Tivoli Directory Server's LDAP Controls
 * via Java and JNDI.
 * 
 * @author Stefan Zoerner
 */
public class PasswordPolicyResponseControl extends PasswordPolicyControl {

    public static final int ERROR_NONE = -1;

    public static final int WARNINGS_DEFAULT = -1;

    private byte[] encodedValue;

    private int errorCode = ERROR_NONE;

    private int timeBeforeExpiration = WARNINGS_DEFAULT;

    private int graceLoginsRemaining = WARNINGS_DEFAULT;

    private static final String[] errorText = { "password expired", "account locked",
            "change after reset", "password mod not allowed", "must supply old password",
            "invalid password syntax", "password too short", "password too young",
            "password in history" };

    /**
     * Returns the graceLoginsRemaining.
     * 
     * @return Returns the graceLoginsRemaining.
     */
    public int getGraceLoginsRemaining() {
        return graceLoginsRemaining;
    }

    /**
     * Sets the graceLoginsRemaining. To be used by control factories.
     * 
     * @param graceLoginsRemaining
     *            The graceLoginsRemaining to set.
     */
    protected void setGraceLoginsRemaining(int graceLoginsRemaining) {
        this.graceLoginsRemaining = graceLoginsRemaining;
    }

    /**
     * Returns the timeBeforeExpiration.
     * 
     * @return Returns the time before expiration in seconds
     */
    public int getTimeBeforeExpiration() {
        return timeBeforeExpiration;
    }

    /**
     * Sets the timeBeforeExpiration. To be used by control factories.
     * 
     * @param timeBeforeExpiration
     *            The timeBeforeExpiration to set in seconds.
     */
    protected void setTimeBeforeExpiration(int timeBeforeExpiration) {
        this.timeBeforeExpiration = timeBeforeExpiration;
    }

    /**
     * Returns the unchanged value of the response control.
     * 
     * Returns the unchanged value of the response control as byte array.
     */
    public byte[] getEncodedValue() {
        return encodedValue;
    }

    /**
     * Sets the unchanged value of the response control. To be used by control
     * factories.
     * 
     * @param value
     *            unchanged value of the response control
     */
    protected void setEncodedValue(byte[] value) {
        this.encodedValue = value;
    }

    /**
     * Returns the error code, or ERROR_NONE, if no error is present.
     * 
     * @return the error code (0-8), or ERROR_NONE
     */
    public int getErrorCode() {
        return errorCode;
    }

    /**
     * Sets the error code. To be used by control factories.
     * 
     * @param code
     *            the error code (0-8), or ERROR_NONE
     */
    protected void setErrorCode(int code) {
        errorCode = code;
    }

    /**
     * Checks whether an error is present.
     * 
     * @return true, if an error is present
     */
    public boolean hasError() {
        return this.getErrorCode() != ERROR_NONE;
    }

    /**
     * Checks whether a warning is present.
     * 
     * @return true, if a warning is present
     */
    public boolean hasWarning() {
        return graceLoginsRemaining != WARNINGS_DEFAULT
                || timeBeforeExpiration != WARNINGS_DEFAULT;
    }

    /**
     * Create a textual representation containing error and warning messages, if
     * any is present.
     * 
     * @return error and warning messages
     */
    public String toString() {

        StringBuffer sb = new StringBuffer("PasswordPolicyResponseControl");

        if (hasError()) {
            sb.append(", error: " + errorText[errorCode]);
        }
        if (graceLoginsRemaining != WARNINGS_DEFAULT) {
            sb.append(", warning: " + graceLoginsRemaining + " grace logins remain");
        }
        if (timeBeforeExpiration != WARNINGS_DEFAULT) {
            sb.append(", warning: time before expiration is " + timeBeforeExpiration);
        }
        if (!hasError() && !hasWarning()) {
            sb.append(" (no error, no warning)");
        }

        return sb.toString();
    }
}